package cn.iocoder.yudao.module.bpm.controller.admin.oa;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.petition.BpmOAPetitionCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.petition.BpmOAPetitionRespVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAPetitionDO;
import cn.iocoder.yudao.module.bpm.service.oa.BpmOAPetitionServiceImpl;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

/**
 * OA 签呈申请 Controller
 *
 * @author 符溶馨
 */
@Tag(name = "管理后台 - OA 签呈申请")
@RestController
@RequestMapping("/bpm/oa/petition")
@Validated
public class BpmOAPetitionController {

    @Resource
    private BpmOAPetitionServiceImpl petitionService;

    @PostMapping("/create")
    @Operation(summary = "创建签呈申请")
    public CommonResult<Long> createPetition(@Valid @RequestBody BpmOAPetitionCreateReqVO createReqVO) {

        return success(petitionService.createPetition(getLoginUserId(), createReqVO));
    }

    @GetMapping("/get")
    @Operation(summary = "获得签呈申请")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    public CommonResult<BpmOAPetitionRespVO> getPetition(@RequestParam("id") Long id) {

        BpmOAPetitionDO petitionDO = petitionService.getPetition(id);

        return success(BeanUtils.toBean(petitionDO, BpmOAPetitionRespVO.class));
    }

    @GetMapping("/getByProcessInstanceId")
    @Operation(summary = "获得签呈申请")
    @Parameter(name = "processInstanceId", description = "流程实例编号", required = true, example = "1024")
    public CommonResult<BpmOAPetitionRespVO> getByProcessInstanceId(@RequestParam("processInstanceId") String processInstanceId) {

        BpmOAPetitionDO petitionDO = petitionService.getByProcessInstanceId(processInstanceId);

        return success(BeanUtils.toBean(petitionDO, BpmOAPetitionRespVO.class));
    }

}
