package cn.iocoder.yudao.module.bpm.controller.admin.oa;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.refund.BpmOARefundCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.refund.BpmOARefundRespVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOARefundDO;
import cn.iocoder.yudao.module.bpm.service.oa.BpmOARefundService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils.getLoginUserId;

/**
 * OA 退款申请 Controller
 *
 * @author 符溶馨

 */
@Tag(name = "管理后台 - OA 退款申请")
@RestController
@RequestMapping("/bpm/oa/refund")
@Validated
public class BpmOARefundController {

    @Resource
    private BpmOARefundService refundService;

    @PostMapping("/create")
    @Operation(summary = "创建请求申请")
    public CommonResult<Long> createRefund(@Valid @RequestBody BpmOARefundCreateReqVO createReqVO) {

        return success(refundService.createRefund(getLoginUserId(), createReqVO));
    }

    @GetMapping("/get")
    @Operation(summary = "获得退款申请")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    public CommonResult<BpmOARefundRespVO> getRefund(@RequestParam("id") Long id) {

        BpmOARefundDO refund = refundService.getRefund(id);

        return success(BeanUtils.toBean(refund, BpmOARefundRespVO.class));
    }

    @GetMapping("/getByProcessInstanceId")
    @Operation(summary = "获得退款申请")
    @Parameter(name = "processInstanceId", description = "流程实例编号", required = true, example = "1024")
    public CommonResult<BpmOARefundRespVO> getByProcessInstanceId(@RequestParam("processInstanceId") String processInstanceId) {

        BpmOARefundDO refund = refundService.getByProcessInstanceId(processInstanceId);

        return success(BeanUtils.toBean(refund, BpmOARefundRespVO.class));
    }
}
