package cn.iocoder.yudao.module.bpm.dal.dataobject.oa;

import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import lombok.*;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;

/**
 * OA 开票申请 DO
 *
 * @author 符溶馨
 */
@TableName(value ="bpm_oa_invoice", autoResultMap = true)
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class BpmOAInvoiceDO extends BaseDO {

    /**
     * 开票申请表单主键
     */
    @TableId
    private Long id;

    /**
     * 申请人的用户编号
     */
    private Long userId;

    /**
     * 计划开票日期
     */
    private LocalDate invoiceDate;

    /**
     * 关联合同业务编号
     */
    private Long contractId;

    /**
     * 关联合同业务的流程实例编号
     */
    private String contractInstanceId;

    /**
     * 客户编号
     */
    private Long customerId;

    /**
     * 开票主体
     */
    private String invoiceBody;

    /**
     * 发票抬头
     */
    private String invoiceName;

    /**
     * 抬头类型 | 1企业2个人3事业单位
     */
    private Integer invoiceIssue;

    /**
     * 发票类型 | 1普票 2专票
     */
    private Integer invoiceType;

    /**
     * 统一社会信用代码
     */
    private String registerNo;

    /**
     * 开票金额
     */
    private BigDecimal amount;

    /**
     * 开票内容
     */
    private String content;

    /**
     * 开票单位地址
     */
    private String unitAddress;

    /**
     * 开票单位联系电话
     */
    private String unitPhone;

    /**
     * 开票单位开户银行
     */
    private String unitBankName;

    /**
     * 开票单位银行账号
     */
    private String unitBankNo;

    /**
     * 接收发票邮箱
     */
    private String email;

    /**
     * 开票状态 | 0未开票 1已开票
     */
    private Integer status;

    /**
     * 申请的结果
     * 枚举 {@link BpmProcessInstanceResultEnum}
     * 考虑到简单，所以直接复用了 BpmProcessInstanceResultEnum 枚举，也可以自己定义一个枚举哈
     */
    private Integer result;

    /**
     * 对应的流程编号
     * 关联 ProcessInstance 的 id 属性
     */
    private String processInstanceId;

    /**
     * 附件基本信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<UploadUserFile> fileItems ;
}
