package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.hire.BpmOAHireCreateReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAHireDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOAHireMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.service.task.BpmHistoryProcessInstanceService;
import cn.iocoder.yudao.module.bpm.service.task.BpmProcessInstanceService;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_HIRE_NOT_EXISTS;

/**
 * OA 招聘申请 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class BpmOAHireServiceImpl extends BpmOABaseService implements BpmOAHireService{

    /**
     * OA 招聘申请对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_hire_2";

    @Resource
    private BpmOAHireMapper bpmOAHireMapper;

    @Resource
    private BpmProcessInstanceService processInstanceService;

    @Resource
    private BpmHistoryProcessInstanceService historyProcessInstanceService;

    @Override
    public Long createHire(Long userId, BpmOAHireCreateReqVO createReqVO) {

        //插入OA 招聘申请
        BpmOAHireDO bpmOAHireDO = BeanUtils.toBean(createReqVO, BpmOAHireDO.class)
                .setUserId(userId)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        bpmOAHireMapper.insert(bpmOAHireDO);

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceService.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(bpmOAHireDO.getId())));

        // 将工作流的编号，更新到 OA 招聘申请中
        bpmOAHireMapper.updateById(new BpmOAHireDO()
                .setId(bpmOAHireDO.getId())
                .setProcessInstanceId(processInstanceId));

        // 判断是否为重新发起的流程
        if (createReqVO.getProcessInstanceId() != null && createReqVO.getResult() == 3) {

            historyProcessInstanceService.createHistoryProcessInstance(processInstanceId, createReqVO.getProcessInstanceId());
        }

        List<UploadUserFile> fileItems = createReqVO.getFileItems();
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (fileItems != null && !fileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId, fileItems);
        }
        return bpmOAHireDO.getId();
    }

    @Override
    public BpmOAHireDO getHire(Long id) {

        return bpmOAHireMapper.selectById(id);
    }

    @Override
    public BpmOAHireDO getByProcessInstanceId(String processInstanceId) {

        return bpmOAHireMapper.selectOne(BpmOAHireDO::getProcessInstanceId, processInstanceId);
    }

    @Override
    public void updateHireResult(Long id, Integer result) {

        validateLeaveExists(id);
    }

    private void validateLeaveExists(Long id) {
        if (bpmOAHireMapper.selectById(id) == null) {
            throw exception(OA_HIRE_NOT_EXISTS);
        }
    }
}
