package cn.iocoder.yudao.module.bpm.service.oa;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.procure.BpmOAProcureListEditReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.reimbursement.BpmOAReimbursementCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.reimbursement.BpmOAReimbursementRespVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.reimbursement.ReimbursementDTO;
import cn.iocoder.yudao.module.bpm.convert.oa.BpmOAReimbursementConvert;
import cn.iocoder.yudao.module.bpm.dal.dataobject.financialpayment.FinancialPaymentDO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAImprestDO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAProcureDO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAReimbursementDO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAReimbursementItemDO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.task.BpmProcessInstanceExtDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOAImprestMapper;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOAReimbursementItemMapper;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOAReimbursementMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.service.financialpayment.FinancialPaymentService;
import cn.iocoder.yudao.module.bpm.service.task.BpmHistoryProcessInstanceService;
import cn.iocoder.yudao.module.bpm.service.task.BpmProcessInstanceService;
import cn.iocoder.yudao.module.system.api.bank.dto.BankRespDTO;
import cn.iocoder.yudao.module.system.api.dept.DeptApi;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptRespDTO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.flowable.engine.runtime.ProcessInstance;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertSet;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_REIMBURSEMENT_NOT_EXISTS;

/**
 * 功能描述
 * OA 报销申请 Service 实现类
 *
 * @author: yj
 * @date: 2024年02月09日 15:33
 */
@Service
@Validated
public class BpmOAReimbursementServiceImpl extends BpmOABaseService implements BpmOAReimbursementService {

    /**
     * OA 请假对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_reimbursement";

    @Resource
    private BpmOAReimbursementMapper reimbursementMapper;

    @Resource
    private BpmProcessInstanceApi processInstanceApi;

    @Resource
    @Lazy // 解决循环依赖
    private BpmProcessInstanceService bpmProcessInstanceService;

    @Resource
    private BpmOAImprestService bpmOAImprestService;

    @Resource
    private BpmOAImprestMapper bpmOAImprestMapper;

    @Resource
    private BpmOAReimbursementItemMapper reimbursementItemMapper;

    @Resource
    private BpmOAProcureService bpmOAProcureService;

    @Resource
    private DeptApi deptApi;

    @Resource
    private AdminUserApi userApi;

    @Resource
    private FinancialPaymentService financialPaymentService;

    @Resource
    private BpmHistoryProcessInstanceService historyProcessInstanceService;


    @Override
    public Long createReimbursement(Long userId, BpmOAReimbursementCreateReqVO createReqVO) {
        BpmOAReimbursementDO reimbursement = BpmOAReimbursementConvert.INSTANCE.convert(createReqVO).setUserId(userId)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        // 如果是备用金报销，则更新备用金流程status
        if (createReqVO.getImprestId() != null) {

            bpmOAImprestMapper.updateById(new BpmOAImprestDO().setId(createReqVO.getImprestId()).setStatus(BpmOAImprestDO.IN_PROGRESS));
        }else {
            // 清空差额字段
            reimbursement.setDifference(BigDecimal.ZERO);
        }
        // 插入 OA 报销表单
        reimbursementMapper.insert(reimbursement);

        // 同步插入明细数据
        List<BpmOAReimbursementItemDO> createDO = BeanUtils.toBean(createReqVO.getReimbursements(), BpmOAReimbursementItemDO.class);
        createDO.forEach(item -> item.setReimbursementId(reimbursement.getId()));
        reimbursementItemMapper.insertBatch(createDO);

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceApi.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(reimbursement.getId()))).getCheckedData();

        // 将工作流的编号，更新到 OA 报销表单中
        reimbursementMapper.updateById(new BpmOAReimbursementDO().setId(reimbursement.getId()).setProcessInstanceId(processInstanceId));

        // 判断是否为重新发起的流程
        if (createReqVO.getProcessInstanceId() != null && createReqVO.getResult() == 3) {

            historyProcessInstanceService.createHistoryProcessInstance(processInstanceId, createReqVO.getProcessInstanceId());
        }

        //判断是否有 采购报销
        List<Long> procureIds = new ArrayList<>();
        for (BpmOAReimbursementItemDO data : createDO) {

            //报销类别为 采购费时
            if ("4".equals(data.getType())) {

                procureIds.add(data.getProcureId());
            }
        }

        if (!procureIds.isEmpty()) {

            //所有关联的采购申请改为 进行中状态
            bpmOAProcureService.updatePayFlagByIds(
                    new BpmOAProcureListEditReqVO()
                            .setPayFlag(BpmOAProcureDO.IN_PROGRESS)
                            .setProcureIds(procureIds)
            );
        }

        List<UploadUserFile> fileItems = createReqVO.getFileItems();
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (fileItems != null && !fileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId, fileItems);
        }
        return reimbursement.getId();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateReimbursementResult(String processInstanceId, Long id, Integer result) {

        BpmOAReimbursementDO bpmOAReimbursementDO = validateLeaveExists(id);

        // 获取明细数据
        List<BpmOAReimbursementItemDO> reimbursements = getReimbursementItem(id);

        //审核通过 （最后节点）
        if (BpmProcessInstanceResultEnum.APPROVE.getResult().equals(result)) {

            ProcessInstance instance = bpmProcessInstanceService.getProcessInstance(processInstanceId);

            if (instance.isEnded()) {

                //判断是否有采购报销
                List<Long> procureIds = new ArrayList<>();
                for (BpmOAReimbursementItemDO reimbursement : reimbursements) {
                    //报销类别为 采购费时
                    if ("4".equals(reimbursement.getType())) {
                        procureIds.add(reimbursement.getProcureId());
                    }
                }

                if (!procureIds.isEmpty()) {

                    //所有关联的采购申请改为 已支付状态
                    bpmOAProcureService.updatePayFlagByIds(
                            new BpmOAProcureListEditReqVO()
                                    .setPayFlag(BpmOAProcureDO.FLAG_TRUE)
                                    .setProcureIds(procureIds)
                    );
                }

                //判断是否为备用金报销
                int isImprest = 0;
                if (bpmOAReimbursementDO.getImprestId() != null) {

                    // 获取备用金详情
                    BpmOAImprestDO bpmOAImprestDO = bpmOAImprestService.getImprest(bpmOAReimbursementDO.getImprestId());

                    BpmOAImprestDO updateVO = new BpmOAImprestDO()
                            .setId(bpmOAReimbursementDO.getImprestId())
                            .setReimbursedAmount(bpmOAImprestDO.getReimbursedAmount().add(bpmOAReimbursementDO.getTotalMoney()));
                    if (bpmOAReimbursementDO.getDifference().compareTo(BigDecimal.ZERO) <= 0) {

                        //将相应备用金申请状态改为 已报销
                        bpmOAImprestMapper.updateById(updateVO.setStatus(BpmOAImprestDO.FLAG_TRUE));

                        if (bpmOAReimbursementDO.getDifference().compareTo(BigDecimal.ZERO) < 0) {
                            isImprest = 1;
                        }else {
                            isImprest = 2;
                        }
                    }else {
                        //将相应备用金申请状态改为 未报销完毕
                        bpmOAImprestMapper.updateById(updateVO.setStatus(BpmOAImprestDO.NOT_COMPLETED));
                        isImprest = 2;
                    }
                }

                String reason = reimbursements.stream().map(BpmOAReimbursementItemDO::getDetail)
                        .filter(StrUtil::isNotEmpty)
                        .collect(Collectors.joining(","));

                BpmProcessInstanceExtDO processInstance = bpmProcessInstanceService.getProcessInstanceDO(processInstanceId);
                // -- 插入到财务支付表中
                CommonResult<AdminUserRespDTO> user = userApi.getUser(bpmOAReimbursementDO.getUserId());

                // 从缓存中部门所属公司信息
                DeptRespDTO deptRespDTO = deptApi.getCompanyByDept(reimbursements.get(0).getDeptId()).getCheckedData();

                financialPaymentService.save(new FinancialPaymentDO()
                        .setUserId(bpmOAReimbursementDO.getUserId())
                        .setDeptId(user.getData() == null ? null : user.getData().getDeptId())
                        .setProcessInstanceId(bpmOAReimbursementDO.getProcessInstanceId())
                        .setReason(reason)
                        .setObjectId(id)
                        .setType(4)
                        .setStatus(isImprest == 2 ? 4 : 0)
                        .setAmountPayable(isImprest == 1 ? bpmOAReimbursementDO.getDifference().abs() : bpmOAReimbursementDO.getTotalMoney())
                        .setProcessInstanceName(processInstance.getName())
                        .setBeginTime(processInstance.getCreateTime())
                        .setEndTime(processInstance.getEndTime())
                        .setRecipientName(bpmOAReimbursementDO.getNickname())
                        .setCompanyId(deptRespDTO != null ? deptRespDTO.getId() : null)
                );
            }
        }

        // -- 自己取消
        // -- 审核拒绝
        //所有关联的采购申请改为 未支付状态
        if (BpmProcessInstanceResultEnum.REJECT.getResult().equals(result)
                || BpmProcessInstanceResultEnum.CANCEL.getResult().equals(result)
                || BpmProcessInstanceResultEnum.BACK.getResult().equals(result)) {

            if (bpmOAReimbursementDO.getImprestId() != null) {

                Long count = reimbursementMapper.selectCount(new LambdaQueryWrapperX<BpmOAReimbursementDO>()
                        .eq(BpmOAReimbursementDO::getImprestId, bpmOAReimbursementDO.getImprestId())
                        .eq(BpmOAReimbursementDO::getResult, BpmProcessInstanceResultEnum.APPROVE.getResult()));
                if (count > 0L) {
                    //将相应备用金申请状态改为 未报销
                    bpmOAImprestMapper.updateById(new BpmOAImprestDO().setId(bpmOAReimbursementDO.getImprestId()).setStatus(BpmOAImprestDO.NOT_COMPLETED));
                }else {
                    bpmOAImprestMapper.updateById(new BpmOAImprestDO().setId(bpmOAReimbursementDO.getImprestId()).setStatus(BpmOAImprestDO.FLAG_FALSE));
                }
            }

            //判断是否有采购报销
            List<Long> procureIds = new ArrayList<>();
//            List<Reimbursement> reimbursements = bpmOAReimbursementDO.getReimbursements();
//
//            //直接从数据库取出来的List<Reimbursement> 实际上是List<LinkedHashMap>类型 所以不能直接遍历
//            //将list再次转为json串，然后由json串再转为list
//            String json = JsonUtils.toJsonString(reimbursements);
//            reimbursements = JsonUtils.parseArray(json, Reimbursement.class);

            for (BpmOAReimbursementItemDO reimbursement : reimbursements) {

                //报销类别为 采购费时
                if ("4".equals(reimbursement.getType())) {

                    BpmOAProcureDO procureDO = bpmOAProcureService.getOaProcure(reimbursement.getProcureId());
                    if (procureDO != null) {

                        procureIds.add(procureDO.getId());
                    }
                }
            }

            if (!procureIds.isEmpty()) {

                //所有关联的采购申请改为 未支付状态
                bpmOAProcureService.updatePayFlagByIds(
                        new BpmOAProcureListEditReqVO()
                                .setPayFlag(BpmOAProcureDO.FLAG_FALSE)
                                .setProcureIds(procureIds)
                );
            }
        }
        reimbursementMapper.updateById(new BpmOAReimbursementDO().setId(id).setResult(result));
    }

    private BpmOAReimbursementDO validateLeaveExists(Long id) {
        BpmOAReimbursementDO reimbursementDO = reimbursementMapper.selectById(id);
        if (reimbursementDO == null) {
            throw exception(OA_REIMBURSEMENT_NOT_EXISTS);
        }
        return reimbursementDO;
    }

    @Override
    public BpmOAReimbursementDO getReimbursement(Long id) {

        return reimbursementMapper.selectById(id);
    }

    @Override
    public List<BpmOAReimbursementItemDO> getReimbursementItem(Long reimbursementId) {

        return reimbursementItemMapper.selectList(BpmOAReimbursementItemDO::getReimbursementId, reimbursementId);
    }

    @Override
    public BpmOAReimbursementRespVO convert(BpmOAReimbursementDO reimbursementDO) {

        // 获取明细数据
        List<BpmOAReimbursementItemDO> reimbursements = getReimbursementItem(reimbursementDO.getId());

//        List<Reimbursement> reimbursement = reimbursementDO.getReimbursements();
//
//        //直接从数据库取出来的List<Reimbursement> 实际上是List<LinkedHashMap>类型 所以不能直接遍历
//        //将list再次转为json串，然后由json串再转为list
//        String json = JsonUtils.toJsonString(reimbursement);
//        reimbursement = JsonUtils.parseArray(json, Reimbursement.class);

        //针对之前的数据， 没有设置明细所属部门的  手动设置发起人的部门编号
        AdminUserRespDTO userRespDTO = userApi.getUser(reimbursementDO.getUserId()).getCheckedData();
        for (BpmOAReimbursementItemDO data : reimbursements) {

            if (data.getDeptId() != null) {
                break;
            }

            data.setDeptId(userRespDTO.getDeptId());
        }

        //获取部门信息map
        Map<Long, DeptRespDTO> deptMap = deptApi.getDeptMap(convertSet(reimbursements, BpmOAReimbursementItemDO::getDeptId));

        List<ReimbursementDTO> reimbursementDTOS = BeanUtils.toBean(reimbursements, ReimbursementDTO.class);

        BpmOAReimbursementRespVO bpmOAReimbursementRespVO = BeanUtils.toBean(reimbursementDO, BpmOAReimbursementRespVO.class);
        bpmOAReimbursementRespVO.setReimbursements(BpmOAReimbursementConvert.INSTANCE.convertList(reimbursementDTOS, deptMap));  //拼接数据

        return bpmOAReimbursementRespVO;
    }

    @Override
    public BpmOAReimbursementDO getByProcessInstanceId(String processInstanceId) {
        List<BpmOAReimbursementDO> list = reimbursementMapper.selectList(
                new LambdaQueryWrapper<BpmOAReimbursementDO>()
                        .eq(BpmOAReimbursementDO::getProcessInstanceId, processInstanceId));
        if (!list.isEmpty()) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public BpmOAReimbursementDO getReimbursementByImprest(Long imprestId) {

        return reimbursementMapper.selectOne(new LambdaQueryWrapper<BpmOAReimbursementDO>()
                .eq(BpmOAReimbursementDO::getImprestId, imprestId)
                .eq(BpmOAReimbursementDO::getResult, BpmProcessInstanceResultEnum.APPROVE.getResult()));
    }
}
