package cn.iocoder.yudao.module.system.api.rental.dto;

import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.List;

@Schema(description = "管理后台 - 租赁订单押金记录新增/修改 Request VO")
@Data
public class RentalDepositRecordDTO {

    @Schema(description = "租赁订单押金记录表单主键", requiredMode = Schema.RequiredMode.REQUIRED, example = "1")
    private Long id;

    @Schema(description = "订单编号", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotEmpty(message = "订单编号不能为空")
    private String orderNo;

    @Schema(description = "支付方式 | 1支付宝 2微信 3银行卡", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "支付方式不能为空")
    private Integer paymentMethod;

    @Schema(description = "交易订单号", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotEmpty(message = "交易订单号不能为空")
    private String transactionOrderNumber;

    @Schema(description = "银行卡号")
    private String bankNo;

    @Schema(description = "客户银行卡号", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotEmpty(message = "客户银行卡号不能为空")
    private String customerBankNo;

    @Schema(description = "收款/退款金额", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "收款/退款金额不能为空")
    private BigDecimal amount;

    @Schema(description = "收款/退款日期", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "收款/退款日期不能为空")
    private LocalDate paymentDate;

    @Schema(description = "类型 | 1收款 2退款", example = "1")
    private Integer type;

    @Schema(description = "附件信息")
    private List<UploadUserFile> fileItems;

}