package cn.iocoder.yudao.module.smartfactory.api.factoryInfo;


import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.module.smartfactory.api.factoryInfo.dto.FactoryInfoDTO;
import cn.iocoder.yudao.module.smartfactory.enums.ApiConstants;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Collection;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertMap;

@FeignClient(name = ApiConstants.NAME) // TODO 芋艿：fallbackFactory =
@Tag(name = "RPC 服务 - 工厂服务")
public interface FactoryInfoApi {

    String PREFIX = ApiConstants.PREFIX + "/factory-info";

    @GetMapping(PREFIX + "/get")
    @Operation(summary = "获得工厂信息")
    CommonResult<FactoryInfoDTO> getFactoryInfo(@RequestParam("factoryId") Long factoryId);

    @GetMapping(PREFIX + "/get-list")
    @Operation(summary = "获得工厂信息列表")
    CommonResult<List<FactoryInfoDTO>> getFactoryInfoList(@RequestParam("factoryId") Collection<Long> factoryId);

    @GetMapping(PREFIX + "/get-all-list")
    @Operation(summary = "获得工厂信息列表")
    CommonResult<List<FactoryInfoDTO>> getAllList();

    /**
     * 获得工厂信息 Map
     * @param factoryId 工厂编号集合
     * @return 工厂信息 Map
     */
    default Map<Long, FactoryInfoDTO> getFactoryMap(Collection<Long> factoryId) {
        List<FactoryInfoDTO> factoryInfoDOS = getFactoryInfoList(factoryId).getCheckedData();
        return convertMap(factoryInfoDOS, FactoryInfoDTO::getId);
    }
}
