package cn.iocoder.yudao.module.smartfactory.controller.admin.handlinggroupamountspecifications;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.module.smartfactory.controller.admin.handlinggroupamountspecifications.vo.HandlingGroupAmountSpecificationsBatchCreateVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.handlinggroupamountspecifications.vo.HandlingGroupAmountSpecificationsPageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.handlinggroupamountspecifications.vo.HandlingGroupAmountSpecificationsRespVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.handlinggroupamountspecifications.vo.HandlingGroupAmountSpecificationsSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.handlinggroupamountspecifications.HandlingGroupAmountSpecificationsDO;
import cn.iocoder.yudao.module.smartfactory.service.handlinggroupamountspecifications.HandlingGroupAmountSpecificationsService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

@Tag(name = "管理后台 - 搬运组工资规格关联")
@RestController
@RequestMapping("/smartfactory/handling-group-amount-specifications")
@Validated
public class HandlingGroupAmountSpecificationsController {

    @Resource
    private HandlingGroupAmountSpecificationsService handlingGroupAmountSpecificationsService;

    @PostMapping("/batchCreate")
    @Operation(summary = "创建搬运组工资规格关联")
    @PreAuthorize("@ss.hasPermission('smartfactory:handling-group-amount-specifications:create')")
    public CommonResult<Boolean> batchCreate(@Valid @RequestBody HandlingGroupAmountSpecificationsBatchCreateVO vo) {
        handlingGroupAmountSpecificationsService.batchCreate(vo);
        return success(true);
    }


    @PostMapping("/update")
    @Operation(summary = "更新搬运组工资规格关联")
    @PreAuthorize("@ss.hasPermission('smartfactory:handling-group-amount-specifications:update')")
    public CommonResult<Boolean> update(@Valid @RequestBody HandlingGroupAmountSpecificationsSaveReqVO createReqVO) {
        handlingGroupAmountSpecificationsService.updateHandlingGroupAmountSpecifications(createReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除搬运组工资规格关联")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('smartfactory:handling-group-amount-specifications:delete')")
    public CommonResult<Boolean> deleteHandlingGroupAmountSpecifications(@RequestParam("id") Long id) {
        handlingGroupAmountSpecificationsService.deleteHandlingGroupAmountSpecifications(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得搬运组工资规格关联")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('smartfactory:handling-group-amount-specifications:query')")
    public CommonResult<HandlingGroupAmountSpecificationsRespVO> getHandlingGroupAmountSpecifications(@RequestParam("id") Long id) {
        HandlingGroupAmountSpecificationsDO handlingGroupAmountSpecifications = handlingGroupAmountSpecificationsService.getHandlingGroupAmountSpecifications(id);
        return success(BeanUtils.toBean(handlingGroupAmountSpecifications, HandlingGroupAmountSpecificationsRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得搬运组工资规格关联分页")
    @PreAuthorize("@ss.hasPermission('smartfactory:handling-group-amount-specifications:query')")
    public CommonResult<PageResult<HandlingGroupAmountSpecificationsRespVO>> getHandlingGroupAmountSpecificationsPage(@Valid HandlingGroupAmountSpecificationsPageReqVO pageReqVO) {
        PageResult<HandlingGroupAmountSpecificationsDO> pageResult = handlingGroupAmountSpecificationsService.getHandlingGroupAmountSpecificationsPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, HandlingGroupAmountSpecificationsRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出搬运组工资规格关联 Excel")
    @PreAuthorize("@ss.hasPermission('smartfactory:handling-group-amount-specifications:export')")
    @OperateLog(type = EXPORT)
    public void exportHandlingGroupAmountSpecificationsExcel(@Valid HandlingGroupAmountSpecificationsPageReqVO pageReqVO,
                                                             HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<HandlingGroupAmountSpecificationsDO> list = handlingGroupAmountSpecificationsService.getHandlingGroupAmountSpecificationsPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "搬运组工资规格关联.xls", "数据", HandlingGroupAmountSpecificationsRespVO.class,
                        BeanUtils.toBean(list, HandlingGroupAmountSpecificationsRespVO.class));
    }

}
