package cn.iocoder.yudao.module.smartfactory.controller.admin.handlingspecifications;

import org.springframework.web.bind.annotation.*;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Operation;

import javax.validation.constraints.*;
import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.smartfactory.controller.admin.handlingspecifications.vo.*;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.handlingspecifications.HandlingSpecificationsDO;
import cn.iocoder.yudao.module.smartfactory.service.handlingspecifications.HandlingSpecificationsService;

@Tag(name = "管理后台 - 工厂搬运规格")
@RestController
@RequestMapping("/smartfactory/handling-specifications")
@Validated
public class HandlingSpecificationsController {

    @Resource
    private HandlingSpecificationsService handlingSpecificationsService;

    @PostMapping("/create")
    @Operation(summary = "创建工厂搬运规格")
    @PreAuthorize("@ss.hasPermission('smartfactory:handling-specifications:create')")
    public CommonResult<Long> createHandlingSpecifications(@Valid @RequestBody HandlingSpecificationsSaveReqVO createReqVO) {
        return success(handlingSpecificationsService.createHandlingSpecifications(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新工厂搬运规格")
    @PreAuthorize("@ss.hasPermission('smartfactory:handling-specifications:update')")
    public CommonResult<Boolean> updateHandlingSpecifications(@Valid @RequestBody HandlingSpecificationsSaveReqVO updateReqVO) {
        handlingSpecificationsService.updateHandlingSpecifications(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除工厂搬运规格")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('smartfactory:handling-specifications:delete')")
    public CommonResult<Boolean> deleteHandlingSpecifications(@RequestParam("id") Long id) {
        handlingSpecificationsService.deleteHandlingSpecifications(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得工厂搬运规格")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('smartfactory:handling-specifications:query')")
    public CommonResult<HandlingSpecificationsRespVO> getHandlingSpecifications(@RequestParam("id") Long id) {
        HandlingSpecificationsDO handlingSpecifications = handlingSpecificationsService.getHandlingSpecifications(id);
        return success(BeanUtils.toBean(handlingSpecifications, HandlingSpecificationsRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得工厂搬运规格分页")
    @PreAuthorize("@ss.hasPermission('smartfactory:handling-specifications:query')")
    public CommonResult<PageResult<HandlingSpecificationsRespVO>> getHandlingSpecificationsPage(@Valid HandlingSpecificationsPageReqVO pageReqVO) {
        PageResult<HandlingSpecificationsDO> pageResult = handlingSpecificationsService.getHandlingSpecificationsPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, HandlingSpecificationsRespVO.class));
    }

    @GetMapping("/list")
    @Operation(summary = "获得工厂搬运规格列表")
    public CommonResult<List<HandlingSpecificationsRespVO>> getHandlingSpecificationsList(@Valid HandlingSpecificationsPageReqVO pageReqVO) {
        List<HandlingSpecificationsDO> list = handlingSpecificationsService.getHandlingSpecificationsList(pageReqVO);
        return success(BeanUtils.toBean(list, HandlingSpecificationsRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出工厂搬运规格 Excel")
    @PreAuthorize("@ss.hasPermission('smartfactory:handling-specifications:export')")
    @OperateLog(type = EXPORT)
    public void exportHandlingSpecificationsExcel(@Valid HandlingSpecificationsPageReqVO pageReqVO,
              HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<HandlingSpecificationsDO> list = handlingSpecificationsService.getHandlingSpecificationsPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "工厂搬运规格.xls", "数据", HandlingSpecificationsRespVO.class,
                        BeanUtils.toBean(list, HandlingSpecificationsRespVO.class));
    }

}
