package cn.iocoder.yudao.module.smartfactory.service.accessoriescollar;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.smartfactory.controller.admin.accessoriescollar.vo.AccessoriesCollarPageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.accessoriescollar.vo.AccessoriesCollarSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.accessoriescollar.AccessoriesCollarDO;
import cn.iocoder.yudao.module.smartfactory.dal.mysql.accessoriescollar.AccessoriesCollarMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.smartfactory.enums.ErrorCodeConstants.ACCESSORIES_COLLAR_NOT_EXISTS;

/**
 * 厂区配件领用 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class AccessoriesCollarServiceImpl implements AccessoriesCollarService {

    @Resource
    private AccessoriesCollarMapper accessoriesCollarMapper;

    @Override
    public Long createAccessoriesCollar(AccessoriesCollarSaveReqVO createReqVO) {
        // 插入
        AccessoriesCollarDO accessoriesCollar = BeanUtils.toBean(createReqVO, AccessoriesCollarDO.class);
        accessoriesCollarMapper.insert(accessoriesCollar);
        // 返回
        return accessoriesCollar.getId();
    }

    @Override
    public void updateAccessoriesCollar(AccessoriesCollarSaveReqVO updateReqVO) {
        // 校验存在
        validateAccessoriesCollarExists(updateReqVO.getId());
        // 更新
        AccessoriesCollarDO updateObj = BeanUtils.toBean(updateReqVO, AccessoriesCollarDO.class);
        accessoriesCollarMapper.updateById(updateObj);
    }

    @Override
    public void deleteAccessoriesCollar(Long id) {
        // 校验存在
        validateAccessoriesCollarExists(id);
        // 删除
        accessoriesCollarMapper.deleteById(id);
    }

    private void validateAccessoriesCollarExists(Long id) {
        if (accessoriesCollarMapper.selectById(id) == null) {
            throw exception(ACCESSORIES_COLLAR_NOT_EXISTS);
        }
    }

    @Override
    public AccessoriesCollarDO getAccessoriesCollar(Long id) {
        return accessoriesCollarMapper.selectById(id);
    }

    @Override
    public PageResult<AccessoriesCollarDO> getAccessoriesCollarPage(AccessoriesCollarPageReqVO pageReqVO) {
        return accessoriesCollarMapper.selectPage(pageReqVO);
    }

}