package cn.iocoder.yudao.module.smartfactory.service.factorypackingline;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorypackingline.vo.FactoryPackingLinePageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorypackingline.vo.FactoryPackingLineRespVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.factorypackingline.vo.FactoryPackingLineSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.factorypackingline.FactoryPackingLineDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 打包线配置 Service 接口
 *
 * @author 姚君
 */
public interface FactoryPackingLineService {

    /**
     * 创建打包线配置
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createFactoryPackingLine(@Valid FactoryPackingLineSaveReqVO createReqVO);

    /**
     * 更新打包线配置
     *
     * @param updateReqVO 更新信息
     */
    void updateFactoryPackingLine(@Valid FactoryPackingLineSaveReqVO updateReqVO);

    /**
     * 删除打包线配置
     *
     * @param id 编号
     */
    void deleteFactoryPackingLine(Long id);

    /**
     * 获得打包线配置
     *
     * @param id 编号
     * @return 打包线配置
     */
    FactoryPackingLineDO getFactoryPackingLine(Long id);

    /**
     * 获得打包线配置分页
     *
     * @param pageReqVO 分页查询
     * @return 打包线配置分页
     */
    PageResult<FactoryPackingLineRespVO> getFactoryPackingLinePage(FactoryPackingLinePageReqVO pageReqVO);

    /**
     * 通过工厂ids获取打包线列表
     *
     * @param factoryIds
     * @return
     */
    List<FactoryPackingLineDO> getFactoryPackingLineByFactoryId(List<Long> factoryIds);
}
