package cn.iocoder.yudao.module.smartfactory.service.handlinggroupamountspecifications;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.module.smartfactory.controller.admin.handlinggroupamountspecifications.vo.HandlingGroupAmountSpecificationsBatchCreateItemVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.handlinggroupamountspecifications.vo.HandlingGroupAmountSpecificationsBatchCreateVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.handlinggroupamountspecifications.vo.HandlingGroupAmountSpecificationsPageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.handlinggroupamountspecifications.vo.HandlingGroupAmountSpecificationsSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.handlinggroupamountspecifications.HandlingGroupAmountSpecificationsDO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.handlingspecifications.HandlingSpecificationsDO;
import cn.iocoder.yudao.module.smartfactory.dal.mysql.handlinggroupamountspecifications.HandlingGroupAmountSpecificationsMapper;
import cn.iocoder.yudao.module.smartfactory.service.handlingspecifications.HandlingSpecificationsService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.smartfactory.enums.ErrorCodeConstants.CANNOT_FIND_THE_CORRESPONDING_FACTORY_HANDLING_SPECIFICATION;

/**
 * 搬运组工资规格关联 Service 实现类
 *
 * @author 艾楷
 */
@Service
@Validated
public class HandlingGroupAmountSpecificationsServiceImpl implements HandlingGroupAmountSpecificationsService {

    @Resource
    private HandlingGroupAmountSpecificationsMapper handlingGroupAmountSpecificationsMapper;
    @Resource
    private HandlingSpecificationsService handlingSpecificationsService;
    private static final BigDecimal THOUSANDTH = new BigDecimal("0.001");

    @Override
    public Long createHandlingGroupAmountSpecifications(HandlingGroupAmountSpecificationsSaveReqVO createReqVO) {
        // 插入
        HandlingGroupAmountSpecificationsDO handlingGroupAmountSpecifications = BeanUtils.toBean(createReqVO, HandlingGroupAmountSpecificationsDO.class);
        handlingGroupAmountSpecificationsMapper.insert(handlingGroupAmountSpecifications);

        // 返回
        return handlingGroupAmountSpecifications.getId();
    }

    @Override
    public void updateHandlingGroupAmountSpecifications(HandlingGroupAmountSpecificationsSaveReqVO updateReqVO) {
        // 更新
        HandlingGroupAmountSpecificationsDO updateObj = BeanUtils.toBean(updateReqVO, HandlingGroupAmountSpecificationsDO.class);
        HandlingSpecificationsDO handlingSpecificationsDO = handlingSpecificationsService.getById(updateObj.getHandlingSpecificationsId());
        if (handlingSpecificationsDO == null) {
            throw exception(CANNOT_FIND_THE_CORRESPONDING_FACTORY_HANDLING_SPECIFICATION);
        }
        BigDecimal totalAmount;
        if (handlingSpecificationsDO.getType() == 0) {
            totalAmount = new BigDecimal(updateReqVO.getTotalCount().toString()).multiply(handlingSpecificationsDO.getUnitPrice());
        } else {
            totalAmount = new BigDecimal(updateReqVO.getTotalCount().toString()).multiply(handlingSpecificationsDO.getWeight())
                    .multiply(handlingSpecificationsDO.getUnitPrice().multiply(THOUSANDTH));
        }
        // 插入
        HandlingGroupAmountSpecificationsDO handlingGroupAmountSpecifications = BeanUtils.toBean(updateReqVO, HandlingGroupAmountSpecificationsDO.class);
        handlingGroupAmountSpecifications.setAmount(totalAmount);
        handlingGroupAmountSpecificationsMapper.updateById(updateObj);
    }

    @Override
    public void deleteHandlingGroupAmountSpecifications(Long id) {
        // 删除
        handlingGroupAmountSpecificationsMapper.deleteById(id);
    }


    @Override
    public HandlingGroupAmountSpecificationsDO getHandlingGroupAmountSpecifications(Long id) {
        return handlingGroupAmountSpecificationsMapper.getHandlingGroupAmountSpecifications(id);
    }

    @Override
    public PageResult<HandlingGroupAmountSpecificationsDO> getHandlingGroupAmountSpecificationsPage(HandlingGroupAmountSpecificationsPageReqVO pageReqVO) {
        IPage<HandlingGroupAmountSpecificationsDO> pageResult = handlingGroupAmountSpecificationsMapper.getHandlingGroupAmountSpecificationsPage(MyBatisUtils.buildPage(pageReqVO), pageReqVO);
        return new PageResult<>(pageResult.getRecords(), pageResult.getTotal());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createOrEdit(HandlingGroupAmountSpecificationsSaveReqVO createReqVO) {
        HandlingSpecificationsDO handlingSpecificationsDO = handlingSpecificationsService.getById(createReqVO.getHandlingSpecificationsId());
        if (handlingSpecificationsDO == null) {
            throw exception(CANNOT_FIND_THE_CORRESPONDING_FACTORY_HANDLING_SPECIFICATION);
        }
        BigDecimal totalAmount;
        if (handlingSpecificationsDO.getType() == 0) {
            totalAmount = new BigDecimal(createReqVO.getTotalCount().toString()).multiply(handlingSpecificationsDO.getUnitPrice());
        } else {
            totalAmount = new BigDecimal(createReqVO.getTotalCount().toString()).multiply(handlingSpecificationsDO.getWeight())
                    .multiply(handlingSpecificationsDO.getUnitPrice().multiply(THOUSANDTH));
        }
        // 插入
        HandlingGroupAmountSpecificationsDO handlingGroupAmountSpecifications = BeanUtils.toBean(createReqVO, HandlingGroupAmountSpecificationsDO.class);
        handlingGroupAmountSpecifications.setAmount(totalAmount);
        handlingGroupAmountSpecificationsMapper.insertOrUpdate(handlingGroupAmountSpecifications);
    }

    @Override
    public void batchCreate(HandlingGroupAmountSpecificationsBatchCreateVO vo) {
        List<HandlingGroupAmountSpecificationsDO> list = new ArrayList<>();
        List<Long> handlingSpecificationsIds = vo.getItemVOS().stream().map(HandlingGroupAmountSpecificationsBatchCreateItemVO::getHandlingSpecificationsId).collect(Collectors.toList());
        List<HandlingSpecificationsDO> handlingSpecificationsDOS = handlingSpecificationsService.list(new LambdaQueryWrapper<HandlingSpecificationsDO>()
                .in(HandlingSpecificationsDO::getId, handlingSpecificationsIds));
        Map<Long, HandlingSpecificationsDO> map = handlingSpecificationsDOS.stream().collect(Collectors.toMap(HandlingSpecificationsDO::getId, item -> item));
        for (HandlingGroupAmountSpecificationsBatchCreateItemVO itemVO : vo.getItemVOS()) {
            HandlingGroupAmountSpecificationsDO handlingGroupAmountSpecificationsDO = new HandlingGroupAmountSpecificationsDO();
            BigDecimal totalAmount;
            HandlingSpecificationsDO handlingSpecificationsDO = map.get(itemVO.getHandlingSpecificationsId());
            if (handlingSpecificationsDO == null) {
                continue;
            }
            if (handlingSpecificationsDO.getType() == 0) {
                totalAmount = new BigDecimal(itemVO.getTotalCount().toString()).multiply(handlingSpecificationsDO.getUnitPrice());
            } else {
                totalAmount = new BigDecimal(itemVO.getTotalCount().toString()).multiply(handlingSpecificationsDO.getWeight())
                        .multiply(handlingSpecificationsDO.getUnitPrice().multiply(THOUSANDTH));
            }
            handlingGroupAmountSpecificationsDO.setFactoryId(vo.getFactoryId());
            handlingGroupAmountSpecificationsDO.setDateStr(vo.getDateStr());
            handlingGroupAmountSpecificationsDO.setHandlingSpecificationsId(itemVO.getHandlingSpecificationsId());
            handlingGroupAmountSpecificationsDO.setAmount(totalAmount);
            handlingGroupAmountSpecificationsDO.setTotalCount(itemVO.getTotalCount());
            list.add(handlingGroupAmountSpecificationsDO);
        }
        handlingGroupAmountSpecificationsMapper.insertBatch(list);
    }
}
