package cn.iocoder.yudao.module.smartfactory.service.handlinggroupuseramount;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.module.smartfactory.controller.admin.handlinggroupuseramount.dto.StaffWagesPageDTO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.handlinggroupuseramount.vo.*;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.handlinggroupamountspecifications.HandlingGroupAmountSpecificationsDO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.handlinggroupuseramount.HandlingGroupUserAmountDO;
import cn.iocoder.yudao.module.smartfactory.dal.mysql.handlinggroupamountspecifications.HandlingGroupAmountSpecificationsMapper;
import cn.iocoder.yudao.module.smartfactory.dal.mysql.handlinggroupuseramount.HandlingGroupUserAmountMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 搬运组每日个人工资记录 Service 实现类
 *
 * @author 艾楷
 */
@Service
@Validated
public class HandlingGroupUserAmountServiceImpl implements HandlingGroupUserAmountService {

    @Resource
    private HandlingGroupUserAmountMapper handlingGroupUserAmountMapper;
    @Resource
    private HandlingGroupAmountSpecificationsMapper handlingGroupAmountSpecificationsMapper;
    private static final String NAME = "姓名";
    private static final String TOTAL_AMOUNT = "总金额";

    @Override
    public Long createHandlingGroupUserAmount(HandlingGroupUserAmountSaveReqVO createReqVO) {
        // 插入
        HandlingGroupUserAmountDO handlingGroupUserAmount = BeanUtils.toBean(createReqVO, HandlingGroupUserAmountDO.class);
        handlingGroupUserAmountMapper.insert(handlingGroupUserAmount);
        // 返回
        return handlingGroupUserAmount.getId();
    }

    @Override
    public void updateHandlingGroupUserAmount(HandlingGroupUserAmountSaveReqVO updateReqVO) {
        // 更新
        HandlingGroupUserAmountDO updateObj = BeanUtils.toBean(updateReqVO, HandlingGroupUserAmountDO.class);
        handlingGroupUserAmountMapper.updateById(updateObj);
    }

    @Override
    public void deleteHandlingGroupUserAmount(Long id) {
        // 删除
        handlingGroupUserAmountMapper.deleteById(id);
    }

    @Override
    public HandlingGroupUserAmountDO getHandlingGroupUserAmount(Long id) {
        return handlingGroupUserAmountMapper.selectById(id);
    }

    @Override
    public PageResult<HandlingGroupUserAmountDO> getHandlingGroupUserAmountPage(HandlingGroupUserAmountPageReqVO pageReqVO) {
        IPage<HandlingGroupUserAmountDO> page = handlingGroupUserAmountMapper.getHandlingGroupUserAmountPage(MyBatisUtils.buildPage(pageReqVO), pageReqVO);
        return new PageResult<>(page.getRecords(), page.getTotal());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createOrEdit(HandlingGroupUserAmountSaveReqVO createReqVO) {
        HandlingGroupAmountSpecificationsDO handlingGroupAmountSpecificationsDO = handlingGroupAmountSpecificationsMapper.selectById(createReqVO.getHandlingGroupAmountSpecificationsId());
        // 插入
        createReqVO.setFactoryId(handlingGroupAmountSpecificationsDO.getFactoryId());
        HandlingGroupUserAmountDO handlingGroupUserAmount = BeanUtils.toBean(createReqVO, HandlingGroupUserAmountDO.class);
        handlingGroupUserAmountMapper.insertOrUpdate(handlingGroupUserAmount);
    }

    @Override
    public PageResult<StaffWagesPageVO> staffWagesPage(StaffWagesPageDTO dto) {
        IPage<StaffWagesPageVO> page = handlingGroupUserAmountMapper.staffWagesPage(MyBatisUtils.buildPage(dto), dto);
        return new PageResult<>(page.getRecords(), page.getTotal());
    }

    @Override
    public HandlingGroupUserAmountMonthVO getListByMonth(Long id, Long factoryId, String time, String name) {
        HandlingGroupUserAmountMonthVO vo = new HandlingGroupUserAmountMonthVO();
        Map<Integer, String> headers = new HashMap<>();
        // -- 根据yyyy-MM获取到该月第一天时间 - 再获取到最后一天 - 完了后组装表头
        String[] times = time.split("-");
        LocalDateTime startTime = LocalDateTime.of(Integer.parseInt(times[0]), Integer.parseInt(times[1]), 1, 0, 0, 0);
        LocalDateTime endTime = startTime.plusMonths(1).minusDays(1);
        List<String> dateList = DateUtils.betweenDayList(startTime, endTime);
        headers.put(1, NAME);
        int num = 1;
        for (String date : dateList) {
            num++;
            headers.put(num, date);
        }
        headers.put(num + 1, TOTAL_AMOUNT);
        // - 插入表头
        vo.setHeaders(headers);

        List<Map<String, Object>> map = handlingGroupUserAmountMapper.getListByMonth(id, factoryId, dateList, name);
        List<HandlingGroupUserAmountMonthItemVO> list = new ArrayList<>();
        for (Map<String, Object> item : map) {
            HandlingGroupUserAmountMonthItemVO itemVO = new HandlingGroupUserAmountMonthItemVO();
            itemVO.setUserId((Long) item.get("userId"));
            Map<Integer, String> itemMap = new HashMap<>();
            itemMap.put(1, item.get("nickName").toString());
            num = 1;
            for (String date : dateList) {
                num++;
                itemMap.put(num, item.get(date).toString());
            }
            itemMap.put(num + 1, item.get("totalAmount").toString());
            itemVO.setContentList(itemMap);
            list.add(itemVO);
        }
        vo.setList(list);
        return vo;
    }


}
