package cn.iocoder.yudao.module.smartfactory.service.staff;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.smartfactory.controller.admin.screendata.factory.vo.StaffDataRespVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.staff.vo.StaffImportExcelVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.staff.vo.StaffImportRespVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.staff.vo.StaffPageReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.staff.vo.StaffSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.staff.StaffDO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 员工 Service 接口
 *
 * @author 艾楷
 */
public interface StaffService {

    /**
     * 创建员工
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createStaff(@Valid StaffSaveReqVO createReqVO);

    /**
     * 更新员工
     *
     * @param updateReqVO 更新信息
     */
    void updateStaff(@Valid StaffSaveReqVO updateReqVO);

    /**
     * 删除员工
     *
     * @param id 编号
     */
    void deleteStaff(Long id);

    /**
     * 获得员工
     *
     * @param id 编号
     * @return 员工
     */
    StaffDO getStaff(Long id);

    /**
     * 获得员工分页
     *
     * @param pageReqVO 分页查询
     * @return 员工分页
     */
    PageResult<StaffDO> getStaffPage(StaffPageReqVO pageReqVO);

    /**
     * 查询工厂员工信息
     *
     * @param factoryId
     * @return
     */
    StaffDataRespVO getStaffData(Long factoryId);

    /**
     * 获取人员数量
     * @return
     */
    Integer getCount();

    /**
     * 查询工厂员工信息
     * @param factoryId 工厂编号
     * @return
     */
    StaffDataRespVO getStaffDataV2(Long factoryId);

    /**
     * 导入员工
     * @param list 导入数据集合
     * @param updateSupport 是否可更新
     * @return 导入结果
     */
    StaffImportRespVO importUserList(List<StaffImportExcelVO> list, Boolean updateSupport);

    /**
     * 获取指定厂区得员工列表
     *
     * @param factoryId 工厂编号
     * @param isIn 是否获取该厂区员工
     * @return 员工列表
     */
    List<StaffDO> getListByFactory(Long factoryId, Boolean isIn);

    /**
     * 获取指定厂区得员工列表
     *
     * @param factoryIds 工厂编号集合
     * @return 员工列表
     */
    List<StaffDO> getListByFactoryIds(Collection<Long> factoryIds);

    /**
     * 获取员工信息列表
     * @param staffIds 员工编号集合
     * @return 员工信息列表
     */
    List<StaffDO> getList(Collection<Long> staffIds);

    /**
     * 获得指定工厂员工
     *
     * @param factoryId
     * @param nickName
     * @return
     */
    List<StaffDO> getListByFactoryId(Long factoryId, String nickName);

    /**
     *
     *
     * @return
     */
    Long getTheFactoryOfTheCurrentlyLoggedInUser();

    /**
     * 根据系统用户编号获取员工信息
     * @param userId 系统用户编号
     * @return 员工信息
     */
    StaffDO getStaffByUserId(Long userId);
}
